package flink.zanxiangnet.ad.monitoring;

import com.tencent.ads.model.DailyReportsGetListStruct;
import com.tencent.ads.model.HourlyReportsGetListStruct;
import flink.zanxiangnet.ad.monitoring.maxcompute.sink.TunnelBatchSink;
import flink.zanxiangnet.ad.monitoring.maxcompute.sink.TunnelStreamSink;
import flink.zanxiangnet.ad.monitoring.pojo.dto.AdDataOfDayDTO;
import flink.zanxiangnet.ad.monitoring.pojo.entity.AdDataOfDayODS;
import flink.zanxiangnet.ad.monitoring.pojo.properties.KafkaProperties;
import flink.zanxiangnet.ad.monitoring.pojo.dto.AdDataOfMinuteDTO;
import flink.zanxiangnet.ad.monitoring.pojo.entity.AdDataOfMinuteODS;
import flink.zanxiangnet.ad.monitoring.util.DateUtil;
import flink.zanxiangnet.ad.monitoring.util.JsonUtil;
import org.apache.commons.lang3.StringUtils;
import org.apache.flink.api.common.eventtime.SerializableTimestampAssigner;
import org.apache.flink.api.common.eventtime.WatermarkStrategy;
import org.apache.flink.api.common.serialization.SimpleStringSchema;
import org.apache.flink.configuration.Configuration;
import org.apache.flink.connector.kafka.source.KafkaSource;
import org.apache.flink.connector.kafka.source.enumerator.initializer.OffsetsInitializer;
import org.apache.flink.streaming.api.datastream.DataStreamSource;
import org.apache.flink.streaming.api.datastream.SingleOutputStreamOperator;
import org.apache.flink.streaming.api.environment.StreamExecutionEnvironment;
import org.apache.kafka.clients.CommonClientConfigs;
import org.apache.kafka.clients.consumer.OffsetResetStrategy;
import org.apache.kafka.common.config.SaslConfigs;
import org.apache.kafka.common.config.SslConfigs;
import org.springframework.beans.BeanUtils;

import java.time.Duration;
import java.time.LocalDateTime;
import java.util.Date;
import java.util.Properties;

public class KafkaDemoJob {

    public static void main(String[] args) throws Exception {
        System.setProperty("javax.net.ssl.trustStore", "D:\\Downloads\\kafka.client.truststore.jks");
        // System.setProperty("javax.net.ssl.trustStore", "/root/flink-1.13.2/kafka.client.truststore.jks");
        System.setProperty("javax.net.ssl.trustStorePassword", "KafkaOnsClient");
        StreamExecutionEnvironment env = StreamExecutionEnvironment.getExecutionEnvironment();

        // 加载配置文件到 flink的全局配置中
        Properties props = new Properties();
        props.load(KafkaDemoJob.class.getResourceAsStream("/application.properties"));
        Configuration configuration = new Configuration();
        props.stringPropertyNames().forEach(key -> {
            String value = props.getProperty(key);
            configuration.setString(key.trim(), StringUtils.isBlank(value) ? "" : value.trim());
        });
        env.getConfig().setGlobalJobParameters(configuration);

        Properties adStreamOfMinuteProps = new Properties();
        adStreamOfMinuteProps.load(KafkaDemoJob.class.getResourceAsStream("/ad_stream_of_minute.properties"));
        KafkaSource<String> adStreamOfMinuteSource = buildKafkaSource(adStreamOfMinuteProps);

        DataStreamSource<String> adStreamOfMinuteIn = env.fromSource(adStreamOfMinuteSource, WatermarkStrategy.noWatermarks(), "adStreamOfMinuteSource_kafka");

        SingleOutputStreamOperator<AdDataOfMinuteODS> adMinuteOdsStream = adStreamOfMinuteIn.filter(StringUtils::isNotBlank)
                .map(str -> {
                    AdDataOfMinuteDTO dto = JsonUtil.toObj(str, AdDataOfMinuteDTO.class);
                    LocalDateTime statTime = DateUtil.milliToLocalDateTime(dto.getCreateTime());
                    HourlyReportsGetListStruct struct = dto.getHourlyReportsGetListStruct();
                    AdDataOfMinuteODS adOds = new AdDataOfMinuteODS();
                    BeanUtils.copyProperties(struct, adOds);
                    adOds.setStatDay(DateUtil.formatLocalDate(statTime.toLocalDate()));
                    adOds.setHour(dto.getHourlyReportsGetListStruct().getHour().intValue());
                    adOds.setStatTime(dto.getCreateTime());
                    adOds.setAccountId(dto.getAccountId());
                    adOds.setAgencyAccountId(dto.getHourlyReportsGetListStruct().getAccountId());
                    return adOds;
                })
                // 打水印，延迟 12分钟，同时指定时间流
                .assignTimestampsAndWatermarks(WatermarkStrategy.<AdDataOfMinuteODS>forBoundedOutOfOrderness(Duration.ofMinutes(12L))
                        .withTimestampAssigner((SerializableTimestampAssigner<AdDataOfMinuteODS>) (adOds, l) -> adOds.getStatTime())
                );
        adMinuteOdsStream.addSink(new TunnelBatchSink<>(AdDataOfMinuteODS.class, 36000L, 64000L, 3));

        Properties adStreamOfDayProps = new Properties();
        adStreamOfDayProps.load(KafkaDemoJob.class.getResourceAsStream("/ad_stream_of_day.properties"));
        KafkaSource<String> adStreamOfDaySource = buildKafkaSource(adStreamOfDayProps);

        DataStreamSource<String> adStreamOfDayIn = env.fromSource(adStreamOfDaySource, WatermarkStrategy.noWatermarks(), "adStreamOfMinuteSource_kafka");

        SingleOutputStreamOperator<AdDataOfDayODS> adDayOdsStream = adStreamOfDayIn.filter(StringUtils::isNotBlank)
                .map(str -> {
                    AdDataOfDayDTO dto = JsonUtil.toObj(str, AdDataOfDayDTO.class);
                    Date createTime = new Date(dto.getCreateTime());
                    DailyReportsGetListStruct struct = dto.getDailyReportsGetListStruct();
                    AdDataOfDayODS adOds = new AdDataOfDayODS();
                    BeanUtils.copyProperties(struct, adOds);
                    adOds.setStatDay(struct.getDate());
                    adOds.setAccountId(dto.getAccountId());
                    adOds.setCampaignId(struct.getCampaignId());
                    adOds.setAgencyAccountId(struct.getAccountId());
                    adOds.setWechatAccountId(struct.getWechatAccountId());
                    adOds.setAdgroupId(struct.getAdgroupId());
                    adOds.setAdId(struct.getAdId());
                    adOds.setCreateTime(createTime);
                    return adOds;
                });
        adDayOdsStream.addSink(new TunnelBatchSink<>(AdDataOfDayODS.class, 36000L, 6000L, 200));

        // 获取指定广告的历史统计信息（用于统计总的消耗信息等）
        /*SingleOutputStreamOperator<AdStatOfHourDWD> oldAdStream = adOdsStream.map(new RichMapFunction<AdDataOfMinuteODS, AdStatOfHourDWD>() {
            // 在这里初始化 MaxCompute连接实例
            private Odps odps;
            @Override
            public void open(Configuration parameters) throws Exception {
                Account account = new AliyunAccount("LTAI5tFuLw65UsH3tqru2K1h", "p1F8my4ovgcEfs3HVORdmeLlLUUKRp");
                odps = new Odps(account);
                // odps.setEndpoint("http://service.odps.aliyun.com/api");
                odps.setEndpoint("http://service.cn-hangzhou.maxcompute.aliyun.com/api");
                // http://dt.cn-hangzhou.maxcompute.aliyun.com
                odps.setDefaultProject("zx_ad_monitoring");
            }
            @Override
            public AdStatOfHourDWD map(AdDataOfMinuteODS adDataOfMinuteODS) throws Exception {
                Instance i = SQLTask.run(odps, "SELECT * FROM ");
                i.waitForSuccess();
                for(Record record : SQLTask.getResult(i)) {
                    record.get(0);
                }
                return null;
            }
        });*/

        /*adOdsStream.join(oldAdStream)
                .where((KeySelector<AdDataOfMinuteODS, Long>) AdDataOfMinuteODS::getAdId)
                .equalTo((KeySelector<AdStatOfHourDWD, Long>) AdStatOfHourDWD::getAdId)
                .window(TumblingEventTimeWindows.of(Time.minutes(5L)));*/

        // 统计广告小时数据
        /*adOdsStream.map(adOds -> {
                    AdStatOfHourDWD adStatOfHour = new AdStatOfHourDWD();
                    adStatOfHour.setStatDay(adOds.getStatDay());
                    adStatOfHour.setHour(adOds.getHour());
                    adStatOfHour.setAccountId(adOds.getAccountId());
                    adStatOfHour.setCampaignId(adOds.getCampaignId());
                    adStatOfHour.setAgencyAccountId(adOds.getAgencyAccountId());
                    adStatOfHour.setWechatAccountId(adOds.getWechatAccountId());
                    adStatOfHour.setWechatAgencyId(adOds.getWechatAgencyId());
                    adStatOfHour.setAdgroupId(adOds.getAdgroupId());
                    adStatOfHour.setAdId(adOds.getAdId());
                    adStatOfHour.setCostDeviationRateHour(adOds.getCostDeviationRate());
                    adStatOfHour.setCostHour(adOds.getCost());
                    adStatOfHour.setCompensationAmountHour(adOds.getCompensationAmount());
                    adStatOfHour.setViewCountHour(adOds.getViewCount());
                    adStatOfHour.setThousandDisplayPriceHour(adOds.getThousandDisplayPrice());
                    adStatOfHour.setAvgViewPerUserHour(adOds.getAvgViewPerUser());
                    adStatOfHour.setValidClickCountHour(adOds.getValidClickCount());
                    adStatOfHour.setCtrHour(adOds.getCtr());
                    adStatOfHour.setCpcHour(adOds.getCpc());
                    adStatOfHour.setValuableClickCountHour(adOds.getValuableClickCount());
                    adStatOfHour.setValuableClickRateHour(adOds.getValuableClickRate());
                    adStatOfHour.setValuableClickCostHour(adOds.getValuableClickCost());
                    adStatOfHour.setConversionsCountHour(adOds.getConversionsCount());
                    adStatOfHour.setConversionsCostHour(adOds.getConversionsCost());
                    adStatOfHour.setConversionsRateHour(adOds.getConversionsRate());
                    adStatOfHour.setDeepConversionsCountHour(adOds.getDeepConversionsCount());
                    adStatOfHour.setDeepConversionsCostHour(adOds.getDeepConversionsCost());
                    adStatOfHour.setDeepConversionsRateHour(adOds.getDeepConversionsRate());
                    adStatOfHour.setOrderCountHour(adOds.getOrderCount());
                    adStatOfHour.setFirstDayOrderCountHour(adOds.getFirstDayOrderCount());
                    adStatOfHour.setWebOrderCostHour(adOds.getWebOrderCost());
                    adStatOfHour.setOrderRateHour(adOds.getOrderRate());
                    adStatOfHour.setOrderAmountHour(adOds.getOrderAmount());
                    adStatOfHour.setFirstDayOrderAmountHour(adOds.getFirstDayOrderAmount());
                    adStatOfHour.setOrderUnitPriceHour(adOds.getOrderUnitPrice());
                    adStatOfHour.setOrderRoiHour(adOds.getOrderRoi());
                    adStatOfHour.setSignInCountHour(adOds.getSignInCount());
                    adStatOfHour.setScanFollowCountHour(adOds.getScanFollowCount());
                    adStatOfHour.setWechatAppRegisterUvHour(adOds.getWechatAppRegisterUv());
                    adStatOfHour.setWechatMinigameRegisterCostHour(adOds.getWechatMinigameRegisterCost());
                    adStatOfHour.setWechatMinigameRegisterRateHour(adOds.getWechatMinigameRegisterRate());
                    adStatOfHour.setWechatMinigameArpuHour(adOds.getWechatMinigameArpu());
                    adStatOfHour.setWechatMinigameRetentionCountHour(adOds.getWechatMinigameRetentionCount());
                    adStatOfHour.setWechatMinigameCheckoutCountHour(adOds.getWechatMinigameCheckoutCount());
                    adStatOfHour.setWechatMinigameCheckoutAmountHour(adOds.getWechatMinigameCheckoutAmount());
                    adStatOfHour.setOfficialAccountFollowCountHour(adOds.getOfficialAccountFollowCount());
                    adStatOfHour.setOfficialAccountFollowRateHour(adOds.getOfficialAccountFollowRate());
                    adStatOfHour.setOfficialAccountRegisterUserCountHour(adOds.getOfficialAccountRegisterUserCount());
                    adStatOfHour.setOfficialAccountRegisterRateHour(adOds.getOfficialAccountRegisterRate());
                    adStatOfHour.setOfficialAccountRegisterCostHour(adOds.getOfficialAccountRegisterCost());
                    adStatOfHour.setOfficialAccountRegisterAmountHour(adOds.getOfficialAccountRegisterAmount());
                    adStatOfHour.setOfficialAccountRegisterRoiHour(adOds.getOfficialAccountRegisterRoi());
                    adStatOfHour.setOfficialAccountApplyCountHour(adOds.getOfficialAccountApplyCount());
                    adStatOfHour.setOfficialAccountApplyUserCountHour(adOds.getOfficialAccountApplyUserCount());
                    adStatOfHour.setOfficialAccountApplyRateHour(adOds.getOfficialAccountApplyRate());
                    adStatOfHour.setOfficialAccountApplyCostHour(adOds.getOfficialAccountApplyCost());
                    adStatOfHour.setOfficialAccountApplyAmountHour(adOds.getOfficialAccountApplyAmount());
                    adStatOfHour.setOfficialAccountApplyRoiHour(adOds.getOfficialAccountApplyRoi());
                    adStatOfHour.setOfficialAccountOrderCountHour(adOds.getOfficialAccountOrderCount());
                    adStatOfHour.setOfficialAccountFirstDayOrderCountHour(adOds.getOfficialAccountFirstDayOrderCount());
                    adStatOfHour.setOfficialAccountOrderUserCountHour(adOds.getOfficialAccountOrderUserCount());
                    adStatOfHour.setOfficialAccountOrderRateHour(adOds.getOfficialAccountOrderRate());
                    adStatOfHour.setOfficialAccountOrderCostHour(adOds.getOfficialAccountOrderCost());
                    adStatOfHour.setOfficialAccountOrderAmountHour(adOds.getOfficialAccountOrderAmount());
                    adStatOfHour.setOfficialAccountFirstDayOrderAmountHour(adOds.getOfficialAccountFirstDayOrderAmount());
                    adStatOfHour.setOfficialAccountOrderRoiHour(adOds.getOfficialAccountOrderRoi());
                    adStatOfHour.setOfficialAccountConsultCountHour(adOds.getOfficialAccountConsultCount());
                    adStatOfHour.setOfficialAccountReaderCountHour(adOds.getOfficialAccountReaderCount());
                    adStatOfHour.setOfficialAccountCreditApplyUserCountHour(adOds.getOfficialAccountCreditApplyUserCount());
                    adStatOfHour.setOfficialAccountCreditUserCountHour(adOds.getOfficialAccountCreditUserCount());
                    adStatOfHour.setForwardCountHour(adOds.getForwardCount());
                    adStatOfHour.setForwardUserCountHour(adOds.getForwardUserCount());
                    adStatOfHour.setNoInterestCountHour(adOds.getNoInterestCount());
                    return adStatOfHour;
                }).keyBy((KeySelector<AdStatOfHourDWD, Long>) AdStatOfHourDWD::getAdId)
                // 开一个 5分钟的滚动窗口
                .window(TumblingEventTimeWindows.of(Time.minutes(5L)))
                // 数据聚合
                .reduce(new RichReduceFunction<AdStatOfHourDWD>() {
                    @Override
                    public void open(Configuration parameters) throws Exception {
                    }
                    @Override
                    public AdStatOfHourDWD reduce(AdStatOfHourDWD adStatOfHourDWD, AdStatOfHourDWD t1) throws Exception {
                        return null;
                    }
                });*/

        // 广告流
        /*SingleOutputStreamOperator<AdInfo> adStream = in.filter(StringUtils::isNotBlank)
                .map(str -> JsonUtil.toObj(str, AdDataOfMinuteDTO.class))
                .map(dto -> {
                    HourlyReportsGetListStruct struct = dto.getHourlyReportsGetListStruct();
                    StatInfo statInfo = new StatInfo();
                    BeanUtils.copyProperties(struct, statInfo);
                    AdInfo adInfo = new AdInfo();
                    adInfo.setStatTime(dto.getCreateTime());
                    adInfo.setStatDay(DateUtil.milliToLocalDate(dto.getCreateTime()));
                    adInfo.setHour(dto.getHourlyReportsGetListStruct().getHour().intValue());
                    adInfo.setAccountId(dto.getAccountId());
                    adInfo.setCampaignId(struct.getCampaignId());
                    adInfo.setAgencyAccountId(struct.getAccountId());
                    adInfo.setWechatAccountId(struct.getWechatAccountId());
                    adInfo.setWechatAgencyId(struct.getWechatAgencyId());
                    adInfo.setAdgroupId(struct.getAdgroupId());
                    adInfo.setAdId(struct.getAdId());
                    adInfo.setStatInfo(statInfo);
                    return adInfo;
                })
                // 打水印，延迟 12分钟，同时指定时间流
                .assignTimestampsAndWatermarks(WatermarkStrategy.<AdInfo>forBoundedOutOfOrderness(Duration.ofMinutes(12L))
                        .withTimestampAssigner((SerializableTimestampAssigner<AdInfo>) (adInfo, l) -> adInfo.getStatTime())
                );

        adStream.keyBy((KeySelector<AdInfo, Long>) AdInfo::getAdId)
                .window(TumblingEventTimeWindows.of(Time.minutes(5L)))
                .apply(new WindowFunction<AdInfo, AdInfo, Long, TimeWindow>() {
                    @Override
                    public void apply(Long key, TimeWindow timeWindow, Iterable<AdInfo> iterable, Collector<AdInfo> collector) throws Exception {
                        for (AdInfo adInfo : iterable) {
                            collector.collect(adInfo);
                        }
                    }
                });*/

        /*SingleOutputStreamOperator<PlanInfo> planStream = adStream.keyBy((KeySelector<AdInfo, Long>) AdInfo::getCampaignId)
                .map()
                .window(TumblingEventTimeWindows.of(Time.minutes(5L)))
                .reduce(new PlanOfHourReduceFunction()).map();
        planStream.print();*/
        // planStatStream.addSink(new PrintSink<>()).name("msg-print");
        env.execute();
    }

    private static KafkaSource<String> buildKafkaSource(Properties props) {
        return buildKafkaSource(props.getProperty(KafkaProperties.KAFKA_SERVERS),
                props.getProperty(KafkaProperties.KAFKA_USERNAME),
                props.getProperty(KafkaProperties.KAFKA_PASSWORD),
                props.getProperty(KafkaProperties.KAFKA_SSL_PATH),
                props.getProperty(KafkaProperties.KAFKA_TOPIC),
                props.getProperty(KafkaProperties.KAFKA_GROUP_ID)
        );
    }

    private static KafkaSource<String> buildKafkaSource(String servers, String username, String password, String sslPath, String topic, String groupId) {
        Properties props = new Properties();
        props.put(SaslConfigs.SASL_JAAS_CONFIG, "org.apache.kafka.common.security.plain.PlainLoginModule required username=\"" + username + "\" password=\"" + password + "\";");
        props.put(SslConfigs.SSL_TRUSTSTORE_LOCATION_CONFIG, sslPath);
        props.put(SslConfigs.SSL_TRUSTSTORE_PASSWORD_CONFIG, "KafkaOnsClient");
        props.put(CommonClientConfigs.SECURITY_PROTOCOL_CONFIG, "SASL_SSL");
        props.put(SaslConfigs.SASL_MECHANISM, "PLAIN");
        props.put(SslConfigs.SSL_ENDPOINT_IDENTIFICATION_ALGORITHM_CONFIG, "");
        return KafkaSource.<String>builder()
                .setBootstrapServers(servers)
                .setTopics(topic)
                .setGroupId(groupId)
                .setProperties(props)
                .setStartingOffsets(OffsetsInitializer.committedOffsets(OffsetResetStrategy.EARLIEST))
                .setValueOnlyDeserializer(new SimpleStringSchema())
                .build();
    }
}
